@@ -248,6 +248,11 @@ test_that("quosures are not calls", {
   expect_false(is_call(quo()))
 })
 
+test_that("is_call() supports symbol `name`", {
+  expect_true(is_call(quote(foo()), quote(foo)))
+  expect_false(is_call(quote(foo()), quote(bar)))
+})
+
 test_that("is_call() vectorises name", {
   expect_false(is_call(quote(foo::bar), c("fn", "fn2")))
   expect_true(is_call(quote(foo::bar), c("fn", "::")))
@@ -555,6 +560,12 @@ test_that("call_zap_inline() works", {
     call_zap_inline(call),
     quote(function(x = `<int>`) foo(`<int>`))
   )
+
+  call2 <- expr(function(x = NULL) foo(!!(1:2)))
+  call2[[2]]$x <- 1:2
+
+  # No mutation
+  expect_equal(call, call2)
 })
 
 test_that("is_call_simple() works", {
@@ -581,3 +592,22 @@ test_that("call_name() and call_ns() detect `::` calls (#670)", {
   expect_null(call_ns(quote(foo::bar)))
   expect_null(call_ns(quote(foo:::bar)))
 })
+
+test_that("is_call_index() works", {
+  expect_true(is_call_index(quote(a$b(...))))
+  expect_true(is_call_index(quote(a@b$c[[d]](...))))
+  expect_true(is_call_index(quote(a@b$c[[d]](...))))
+  expect_true(is_call_index(quote(foo::a$b(...))))
+
+  expect_false(is_call_index(quote(a@b$c[[d]])))
+  expect_false(is_call_index(quote(1 + a@b$c[[d]])))
+  expect_false(is_call_index(quote((a@b$c[[d]])())))
+})
+
+test_that("call_match() supports `...` in arg list when `dots_expand = FALSE`", {
+  f <- function(x, ...) NULL
+  expect_equal(
+    call_match(quote(f(...)), f, dots_expand = FALSE),
+    quote(f())
+  )
+})
