@@ -1,5 +1,3 @@
-context("quo")
-
 test_that("quo_get_expr() and quo_get_env() retrieve quosure components", {
   quo <- quo(foo)
   expect_identical(quo_get_expr(quo), quote(foo))
@@ -82,14 +80,6 @@ test_that("as_quosure() uses correct env", {
   expect_identical(out_quo$quo, quo)
 })
 
-test_that("explicit promise makes a formula", {
-  capture <- function(x) enquo(x)
-  f1 <- capture(1 + 2 + 3)
-  f2 <- ~ 1 + 2 + 3
-
-  expect_equal(f1, f2)
-})
-
 test_that("explicit promise works only one level deep", {
   f <- function(x) list(env = current_env(), f = g(x))
   g <- function(y) enquo(y)
@@ -147,7 +137,7 @@ test_that("as_quosure() coerces formulas", {
 })
 
 test_that("quo_squash() warns", {
-  expect_warning(regex = NA, quo_squash(quo(foo), warn = TRUE))
+  expect_warning(regexp = NA, quo_squash(quo(foo), warn = TRUE))
   expect_warning(quo_squash(quo(list(!! quo(foo))), warn = TRUE), "inner quosure")
 })
 
@@ -275,7 +265,7 @@ test_that("new_quosure() checks input", {
 })
 
 test_that("as_string(quo) produces informative error message", {
-  expect_error(as_string(quo(foo)), "a `quosure/formula` object to a string")
+  expect_error(as_string(quo(foo)), "a <quosure> object to a string")
 })
 
 test_that("`[` properly reconstructs quosure lists", {
@@ -289,12 +279,33 @@ test_that("quosure lists are considered vectors", {
   expect_identical(vctrs::vec_slice(quos(1, 2, 3), 2:3), quos(2, 3))
 })
 
+test_that("quosure attributes are cloned (#1142)", {
+  x <- quos()
+  attr(x, "foo") <- TRUE
+  y <- quos()
+  expect_true(setequal(names(attributes(y)), c("names", "class")))
+})
+
+test_that("quo_squash() supports nested missing args", {
+  expect_equal(
+    quo_squash(expr(foo(!!quo()))),
+    quote(foo(, ))[1:2]
+  )
+  expect_equal(
+    quo_squash(expr(foo(bar(!!quo(), !!quo())))),
+    quote(foo(bar(, )))
+  )
+
+  expect_equal(quo_squash(missing_arg()), missing_arg())
+  expect_equal(quo_squash(quo()), missing_arg())
+})
+
 
 # Lifecycle ----------------------------------------------------------
 
 test_that("as_quosure() still provides default env", {
   local_lifecycle_warnings()
-  quo <- expect_warning(as_quosure(quote(foo)), "explicit environment")
+  expect_warning(quo <- as_quosure(quote(foo)), "explicit environment")
   expect_reference(quo_get_env(quo), current_env())
 })
 
